#include <apponeservices_adaptor.h>
#include <apptwoservices_interface.h>

#include <QCoreApplication>
#include <QDebug>
#include <QTimer>
#include <QtDBus>

using namespace com::bluescape;

class AppOne : public QObject
{
    Q_OBJECT
public:
    AppOne(QObject* parent = nullptr)
        : QObject(parent) , m_dbusAdaptor(new AppOneAdaptor(this)), m_pingReceived(false), m_pingSent(false)
    {
        QDBusConnection bus(QDBusConnection::sessionBus());
        if (!bus.registerObject("/", this)) {
            qCritical() << "Can't register" << serviceName() << "DBus object! Client calls will fail.";
        }
        if (!bus.registerService(serviceName())) {
            qCritical() << "Can't register" << serviceName() << "DBus service! Client calls will fail.";
        }
    }

    QString serviceName() const { return m_dbusAdaptor->metaObject()->classInfo(m_dbusAdaptor->metaObject()->indexOfClassInfo("D-Bus Interface")).value(); }

    bool pingReceived() const { return m_pingReceived; }
    void pingSent() { m_pingSent = true; }

public slots:
    void ping(const QString &message)
    {
        qDebug() << "AppOne: PING" << message;
        m_pingReceived = true;
        if (m_pingSent)
            QCoreApplication::quit();
    }

private:
    AppOneAdaptor* m_dbusAdaptor;
    bool m_pingReceived;
    bool m_pingSent;
};

int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);

    qDebug() << "AppOne Starting...";
    AppOne appOne;
    qDebug() << "AppOne Started...";

    AppTwo proxy(AppTwo::staticInterfaceName(), "/", QDBusConnection::sessionBus());

    QTimer delayedCallTimer;
    delayedCallTimer.setInterval(100);
    delayedCallTimer.setSingleShot(true);

    auto tryPing = [&appOne, &proxy, &delayedCallTimer]() {
        if (proxy.isValid()) {
            qDebug() << "AppTwo proxy valid, sending ping!";
            proxy.ping("Hello from AppOne!");
            appOne.pingSent();
            if (appOne.pingReceived())
                QCoreApplication::quit();
        } else {
            qDebug() << "Waiting for AppTwo proxy...";
            delayedCallTimer.start();
        }
    };
    QObject::connect(&delayedCallTimer, &QTimer::timeout, tryPing);

    tryPing();

    return app.exec();
}

#include <main.moc>
