#include <apptwoservices_adaptor.h>
#include <apponeservices_interface.h>

#include <QCoreApplication>
#include <QDebug>
#include <QtDBus>

using namespace com::bluescape;

class AppTwo : public QObject
{
    Q_OBJECT
public:
    AppTwo(QObject* parent = nullptr)
        : QObject(parent) , m_dbusAdaptor(new AppTwoAdaptor(this)), m_pingReceived(false), m_pingSent(false)
    {
        QDBusConnection bus(QDBusConnection::sessionBus());
        if (!bus.registerObject("/", this)) {
            qCritical() << "Can't register" << serviceName() << "DBus object! Client calls will fail.";
        }
        if (!bus.registerService(serviceName())) {
            qCritical() << "Can't register" << serviceName() << "DBus service! Client calls will fail.";
        }
    }

    QString serviceName() const { return m_dbusAdaptor->metaObject()->classInfo(m_dbusAdaptor->metaObject()->indexOfClassInfo("D-Bus Interface")).value(); }

    bool pingReceived() const { return m_pingReceived; }
    void pingSent() { m_pingSent = true; }

public slots:
    void ping(const QString &message)
    {
        qDebug() << "AppTwo: PING" << message;
        m_pingReceived = true;
        if (m_pingSent)
            QCoreApplication::quit();
    }

private:
    AppTwoAdaptor* m_dbusAdaptor;
    bool m_pingReceived;
    bool m_pingSent;
};

int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);

    qDebug() << "AppTwo Starting...";

// !!!
// UNCOMMENT THE LINE BELOW TO DELAY STARTUP AND SOLVE THE RANDOM PROXY VALIDITY BUG
//    QThread::msleep(500);
// !!!

    AppTwo appTwo;
    qDebug() << "AppTwo Started...";

    AppOne proxy(AppOne::staticInterfaceName(), "/", QDBusConnection::sessionBus());

    QTimer delayedCallTimer;
    delayedCallTimer.setInterval(100);
    delayedCallTimer.setSingleShot(true);

    auto tryPing = [&appTwo, &proxy, &delayedCallTimer]() {
        if (proxy.isValid()) {
            qDebug() << "AppOne proxy valid, sending ping!";
            proxy.ping("Hello from AppTwo!");
            appTwo.pingSent();
            if (appTwo.pingReceived())
                QCoreApplication::quit();
        } else {
            qDebug() << "Waiting for AppOne proxy...";
            delayedCallTimer.start();
        }
    };
    QObject::connect(&delayedCallTimer, &QTimer::timeout, tryPing);

    tryPing();

    return app.exec();
}

#include <main.moc>
