#include <QDebug>
#include <QGuiApplication>
#include <QProcess>
#include <QWindow>

class ProcessMonitor : public QObject
{
    Q_OBJECT

public:
    ProcessMonitor(QObject* parent = nullptr)
        : QObject(parent)
        , m_appOneProcess(new QProcess(this))
        , m_appTwoProcess(new QProcess(this))
        , m_restartProcesses(true)
    {
        m_appOneProcess->setReadChannel(QProcess::StandardError);
        m_appTwoProcess->setReadChannel(QProcess::StandardError);

        connect(m_appOneProcess, &QProcess::readyRead, this, [this]() {
            while (true) {
                const QByteArray line = m_appOneProcess->readLine();
                if (line.isEmpty())
                    break;
                qDebug() << "[AppOne]" << line.constData();
            }
        });
        connect(m_appTwoProcess, &QProcess::readyRead, this, [this]() {
            while (true) {
                const QByteArray line = m_appTwoProcess->readLine();
                if (line.isEmpty())
                    break;
                qDebug() << "[AppTwo]" << line.constData();
            }
        });

        connect(m_appOneProcess, SIGNAL(finished(int)), this, SLOT(onProcessFinished()));
        connect(m_appTwoProcess, SIGNAL(finished(int)), this, SLOT(onProcessFinished()));

        startProcesses();
    }
    ~ProcessMonitor()
    {
        qDebug() << "Cleaning up processes...";
        m_restartProcesses = false;
        m_appOneProcess->terminate();
        m_appOneProcess->waitForFinished();
        m_appTwoProcess->terminate();
        m_appTwoProcess->waitForFinished();
    }

    void startProcesses()
    {
        m_appOneProcess->start("./AppOne");
        m_appTwoProcess->start("./AppTwo");
    }

public slots:
    void onProcessFinished() {
        if (!m_restartProcesses)
            return;

        if (m_appOneProcess->state() == QProcess::Running)
            qDebug() << "Waiting on AppOne to finish...";
        else if (m_appTwoProcess->state() == QProcess::Running)
            qDebug() << "Waiting on AppTwo to finish...";
        else {
            qDebug() << "All apps finished, restarting processes";
            startProcesses();
        }
    }

private:
    QProcess* m_appOneProcess;
    QProcess* m_appTwoProcess;
    bool m_restartProcesses;
};

int main(int argc, char *argv[])
{
    QGuiApplication app(argc, argv);

    QWindow w;
    w.setTitle("Close to quit app");
    w.show();

    ProcessMonitor monitor;

    return app.exec();
}

#include <main.moc>
